#!/usr/bin/env python

from importlib.resources import path

import numpy as np
import matplotlib.pyplot as plt

from iodata import load_one
from iodata.overlap import compute_overlap

from grid.becke import BeckeWeights
from grid.molgrid import MolGrid
from grid.onedgrid import GaussChebyshev
from grid.rtransform import BeckeTF
from grid.lebedev import n_points

from gbasis.wrappers import from_iodata
from gbasis.evals.density import evaluate_density


def main():
    # This script only works with restricted, just to keep things simple.
    fns_wfn = [
        "he2_ghost_psi4_1.0.molden",
        "monosilicic_acid_hf_lan.fchk",
        "water_dimer_ghost.fchk",
        "hf_sto3g.fchk",
    ]
    nlebs = n_points[:20]
    plt.figure(figsize=(8, 4.5))
    plt.clf()
    for fn_wfn in fns_wfn:
        errors = verify_rho(fn_wfn, nlebs)
        plt.semilogy(np.arange(len(nlebs)), abs(errors), label=fn_wfn)
    plt.axvline(5, color="k")  # 74
    plt.axvline(11, color="k")  # 230
    plt.axvline(12, color="k")  # 266
    plt.xticks(np.arange(len(nlebs)), nlebs)
    plt.ylabel("Density integration error [a.u.]")
    plt.xlabel("Lebedev grid size")
    plt.legend(loc=0)
    plt.tight_layout()
    plt.savefig("lebcheck.png", dpi=200)


def verify_rho(fn_wfn, nlebs):
    """Compute integration errors with different Lebedev grids."""
    print(fn_wfn)
    # Get a test file from IOData
    with path("iodata.test.data", fn_wfn) as fn_full:
        iodata = load_one(str(fn_full))
    olp = compute_overlap(iodata.obasis, iodata.atcoords)
    # Orthogonalization of the orbitals.
    # This is to avoid that round-off errors in
    # the data files result in integration errors.
    if iodata.mo.kind != "restricted":
        raise ValueError("This script only works for resricted Slater dets.")
    if (iodata.mo.occs != np.round(iodata.mo.occs)).all():
        raise ValueError("Fractional occupations are not ideal for this check.")
    mask = iodata.mo.occs != 0
    occs = iodata.mo.occs[mask]
    assert (occs == occs[0]).all()
    coeffs = iodata.mo.coeffs[:, mask]
    evals, evecs = np.linalg.eigh(np.dot(coeffs.T, np.dot(olp, coeffs)))
    coeffs = np.dot(coeffs, evecs) / np.sqrt(evals)
    assert abs(np.dot(coeffs.T, np.dot(olp, coeffs)) - np.identity(mask.sum())).max() < 1e-14

    # Get the rdm
    one_rdm = np.dot(coeffs * occs, coeffs.T)
    # Derive a basis list suitable for gbasis
    basis, coord_types = from_iodata(iodata)

    # Loop over sizes of lebedev grids, keeping the same radial grid.
    becke = BeckeWeights(order=3)
    oned = GaussChebyshev(256)
    errors = []
    for nleb in nlebs:
        print("   ", nleb)
        with np.errstate(all="ignore"):
            rgrid = BeckeTF(1e-4, 1.5).transform_1d_grid(oned)
            grid = MolGrid.horton_molgrid(
                iodata.atcoords, iodata.atnums, rgrid, nleb, becke
            )
        rho = evaluate_density(one_rdm, basis, grid.points, coord_type=coord_types)
        errors.append(grid.integrate(rho) - iodata.mo.nelec)
    return np.array(errors)


if __name__ == "__main__":
    main()
